/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

"use strict";

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");

var m_isInstall = false;

// install & uninstall are called even for disabled extensions
function install(data, reason) {
  m_isInstall = true;
}


function uninstall(data, reason) {
  registerResourceProtocol(data.resourceURI);

  // uninstall: remove storage, prefs
  if (reason === ADDON_UNINSTALL) { // updating=ADDON_UPGRADE
    Bootstrap.extensionUninstall();
  }

  // update/uninstall: unload all modules
  registerResourceProtocol(null);
}


function startup(data, reason) {
  // resource protocol needed because when mm.broadcastAsyncMessage("bbb@hultmann:disable")
  // listener executes chrome protocol doesn't exist anymore
  registerResourceProtocol(data.resourceURI);
  Bootstrap.extensionStartup(m_isInstall);
}


function shutdown(data, reason) {
  if (reason !== APP_SHUTDOWN) {
    Bootstrap.extensionShutdown();
  }
}


// chrome.manifest line:
// resource ext-modules modules/
function registerResourceProtocol(uri) { // null to unregister
  var module = uri ? Services.io.newURI(uri.resolve("content/"), null, null) : null;
  Services.io.getProtocolHandler("resource").
                QueryInterface(Ci.nsIResProtocolHandler).
                  setSubstitution("bbbb-1", module);
}


var Bootstrap = {
  extensionStartup: function(isInstall) {
    if (isInstall) {
      console.log("bbb isInstall setting prefs");
      this._setDefaultPrefs();
    }

    var mm = Cc["@mozilla.org/globalmessagemanager;1"].getService(Ci.nsIMessageListenerManager);
    mm.loadFrameScript("resource://bbbb-1/content.js?v=1", true);
  },


  _setDefaultPrefs: function() {
    function setBool(name, val) {
      if (Services.prefs.getPrefType(name) !== Services.prefs.PREF_BOOL) {
        Services.prefs.setBoolPref(name, val);
      }
    }

    setBool("services.sync.prefs.sync.extensions.bbb@multifox.blockmode", true);
    setBool("services.sync.prefs.sync.extensions.bbb@multifox.social", true);
    setBool("services.sync.prefs.sync.extensions.bbb@multifox.news", true);
    setBool("services.sync.prefs.sync.extensions.bbb@multifox.allsites", true);

    if (Services.prefs.getPrefType("extensions.bbb@multifox.blockmode") !== Services.prefs.PREF_STRING) {
      Services.prefs.setCharPref("extensions.bbb@multifox.blockmode", "blur");
    }

    setBool("extensions.bbb@multifox.social", true);
    setBool("extensions.bbb@multifox.news", true);
    setBool("extensions.bbb@multifox.allsites", false);
  },


  extensionShutdown: function() {
    var mm = Cc["@mozilla.org/globalmessagemanager;1"].getService(Ci.nsIMessageListenerManager);
    mm.addMessageListener("bbb@hultmann:content-disabled", function unload(message) {
      mm.removeMessageListener("bbb@hultmann:content-disabled", unload);
      registerResourceProtocol(null);
      console.log("BBB chrome disable done!");
    });

    mm.removeDelayedFrameScript("resource://bbbb-1/content.js?v=1");
    mm.broadcastAsyncMessage("bbb@hultmann:disable");
  },


  extensionUninstall: function() {
    var branch = "extensions.bbb@multifox.";
    Services.prefs.getBranch(branch).deleteBranch("");
    Services.prefs.getBranch("services.sync.prefs.sync." + branch).deleteBranch("");
  }
};
